@@ -109,14 +109,18 @@ momentEstim.baseGmm.twoStep.formula <- function(object, ...)
   {
   P <- object
   g <- P$g
-  dat <- getDat(P$gform, P$x)
+  if (is.null(P$data))
+    	dat <- getDat(P$gform, P$x)
+    else
+    	dat <- getDat(P$gform, P$x, P$data)
+  
   x <- dat$x
   k <- dat$k
   k2 <- k*dat$ny
   n <- nrow(x)
   q <- dat$ny*dat$nh
   df <- q-k*dat$ny
-  
+    
   if (q == k2 | P$wmatrix == "ident")
     {
     w <- diag(q)
@@ -125,12 +129,14 @@ momentEstim.baseGmm.twoStep.formula <- function(object, ...)
     }
   else
     {
-    w=diag(rep(1, q))
-    res1 <- .tetlin(x, w, dat$ny, dat$nh, dat$k, P$gradv, P$g)
     if (P$vcov == "iid")
-      w <- P$iid(res1$par, x, g, P$centeredVcov)
+    	{
+      res2 <- .tetlin(x, diag(q), dat$ny, dat$nh, dat$k, P$gradv, P$g, type="2sls")
+      }
     if (P$vcov == "HAC")
       {
+      w=diag(q)
+      res1 <- .tetlin(x, w, dat$ny, dat$nh, dat$k, P$gradv, P$g)
       if(P$centeredVcov) 
        	 gmat <- lm(g(res1$par, x)~1)
       else
@@ -140,9 +146,9 @@ momentEstim.baseGmm.twoStep.formula <- function(object, ...)
         }
       w <- kernHAC(gmat, kernel = P$kernel, bw = P$bw, prewhite = P$prewhite, 
 		ar.method = P$ar.method, approx = P$approx, tol = P$tol, sandwich = FALSE)
-
+	 res2 <- .tetlin(x, w, dat$ny, dat$nh, dat$k, P$gradv, g)
       }
-    res2 <- .tetlin(x, w, dat$ny, dat$nh, dat$k, P$gradv, g)
+    
     z = list(coefficients = res2$par, objective = res2$value, dat=dat, k=k, k2=k2, n=n, q=q, df=df)	
     }
   z$gt <- g(z$coefficients, x) 
@@ -186,7 +192,11 @@ momentEstim.baseGmm.iterative.formula <- function(object, ...)
   {
   P <- object
   g <- P$g
-  dat <- getDat(P$gform, P$x)
+  if (is.null(P$data))
+    	dat <- getDat(P$gform, P$x)
+    else
+    	dat <- getDat(P$gform, P$x, P$data)
+  
   x <- dat$x
   k <- dat$k
   k2 <- k*dat$ny
@@ -373,7 +383,11 @@ momentEstim.baseGmm.cue.formula <- function(object, ...)
   {
   P <- object
   g <- P$g
-  dat <- getDat(P$gform, P$x)
+  if (is.null(P$data))
+    	dat <- getDat(P$gform, P$x)
+    else
+    	dat <- getDat(P$gform, P$x, P$data)
+  
   x <- dat$x
   k <- dat$k
   k2 <- k*dat$ny
@@ -729,7 +743,11 @@ momentEstim.fixedW.formula <- function(object, ...)
   {
   P <- object
   g <- P$g
-  dat <- getDat(P$gform, P$x)
+  if (is.null(P$data))
+    	dat <- getDat(P$gform, P$x)
+    else
+    	dat <- getDat(P$gform, P$x, P$data)
+
   x <- dat$x
   k <- dat$k
   k2 <- k*dat$ny
